<?php
/**
 * 2007-2020 PrestaShop and Contributors
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * @author    PayTR Ödeme ve Elektronik Para Kuruluşu A.Ş. <devops@paytr.com>
 * @copyright PayTR Ödeme ve Elektronik Para Kuruluşu A.Ş.
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 * International Registered Trademark & Property of PayTR Ödeme ve Elektronik Para Kuruluşu A.Ş.
 */

require_once(dirname(__FILE__) . '/../../classes/PaytrNotification.php');

class PaytrCheckoutNotificationModuleFrontController extends ModuleFrontController
{
    private string $uniq_separator = 'PAYTRYPS';

    /**
     * Handle PayTR notification
     */
    public function postProcess(): void
    {
        if (!$this->validateNotification()) {
            echo '';
            exit;
        }

        $this->handleNotification();
    }

    /**
     * Validate if the notification is valid
     *
     * @return bool
     */
    private function validateNotification(): bool
    {
        return isset($_POST) && !empty(Tools::getValue('hash'));
    }

    /**
     * Process the notification based on payment type
     */
    private function handleNotification(): void
    {
        $notification = new PaytrNotification();
        $notification->context = Context::getContext();

        $payment_type = Tools::getValue('payment_type');
        $notification->module = $this->getModuleInstance($payment_type);
        $this->executeCallback($notification, $payment_type);
    }

    /**
     * Get the appropriate module instance based on payment type
     *
     * @param string $payment_type Payment type (eft or iframe)
     * @return Module
     */
    private function getModuleInstance(string $payment_type): Module
    {
        return $payment_type === 'eft'
            ? Module::getInstanceByName('paytrefttransfer')
            : $this->module;
    }

    /**
     * Execute the correct callback method for the payment type
     *
     * @param PaytrNotification $notification Notification handler
     * @param string $payment_type Payment type
     */
    private function executeCallback(PaytrNotification $notification, string $payment_type): void
    {
        $method = $payment_type === 'eft' ? 'eftCallback' : 'iframeCallback';
        $notification->$method($_POST, $this->uniq_separator);
    }
}